#include<stdio.h>
#include<stdlib.h>
#include<dos.h>
#include<conio.h>

/* I wrote this one night before an important trip.  I didn't fully
   trust my clock-radio, and I'd used up my download privs for the day
   on the local bulletin board.  And I didn't have alarm programs among
   the megs of useless utils on my hard disk.  So, I wrote a bare-bones
   code which would simply check the time each half-hour, and beep my pc
   speaker with an annoying tone at the specified time.  I've enhanced
   the code somewhat, and figured someone else might need a simple
   wakeup program too.

   The code is extremely easy to use.  Simply specify an hour and
   optional minute and second (the default for the latter two is 0) for
   the alarm to go off.  The alarm works on a 24 hour clock, and the
   alarm time is checked for validity.  For example, to wake up at
   5:35pm, type:

       wakeup 17 35

   The alarm runs in the foreground (you're supposed to be sleeping,
   anyway), and sounds a (somewhat annoying) 440Hz tone every other
   second when the alarm time is reached, until you press a key; the
   keyboard buffer is flushed (thanks to heilig@plains.nodak.edu for the
   code) immediately before the alarm sounds just to keep you safe from
   unexpected keystrokes.  To cancel the alarm before it has reached the
   alarm time, simply hit ctrl-c or ctrl-break.

   This is a very simple code, and so is released into the public domain
   for anyone to use, abuse, and change the alarm to whatever irritating
   noise you want.  This means that if the alarm scares your hamster to
   death, or causes you any longterm brain damage, you can't sue me.

   By the way, this code was compiled with Turbo C++ v1.0.  Microsoft C
   may require some different names for the gettime() and kbhit()
   routines.

   David Steinman (steinman@me.utoronto.ca)
*/

void main(int argc, char *argv[])
{
    struct time t;
    int hour,min,sec;

    /* display syntax if no arguments */
    if (--argc<1) {
        fprintf(stderr,"\n");
        fprintf(stderr,"WAKEUP v1.0 - released into the public domain\n");
        fprintf(stderr,"              by David Steinman\n");
        fprintf(stderr,"\n");
        fprintf(stderr,"     syntax:  wakeup hour [min] [sec]\n");
        fprintf(stderr,"\n");
        fprintf(stderr,"WAKEUP is a simple command-line alarm clock.\n");
        fprintf(stderr,"Just specify the time (in 24 hour format) at\n");
        fprintf(stderr,"which you want the PC to wakeup.  When the\n");
        fprintf(stderr,"alarm sounds, hit any key to stop it.\n");
		exit(1);
	}

    /* set defaults */
    sec = 0;
    min = 0;

    /* get command line arguments */
    hour = atoi(argv[1]);
    if (argc>1) min = atoi(argv[2]);
    if (argc>2) sec = atoi(argv[3]);

    /* check for invalid wakeup time */
    if (hour>23 || hour<0 || min>59 || min<0 || sec>59 || sec<0) {
        fprintf(stderr,"\n");
        fprintf(stderr,"Invalid wakeup time:  %02dh %02dm %02ds\n",hour,min,sec);
        exit(1);
    }

    /* display ongoing status message */
    fprintf(stderr,"\n");
    fprintf(stderr," Wakeup time:  %02dh %02dm %02ds\n",hour,min,sec);
    fprintf(stderr,"Current time:             ");

    /* get time each second until wakeup time */
    do {
        fprintf(stderr,"\b\b\b\b\b\b\b\b\b\b\b");
        delay(999);
        gettime(&t);
        fprintf(stderr,"%02dh %02dm %02ds",t.ti_hour,t.ti_min,t.ti_sec);
    } while (t.ti_hour!=hour || t.ti_min!=min || t.ti_sec!=sec);

    /* flush any pending keystrokes */
    while (kbhit()) getch();

    /* sound alarm until keypress */
    while (!kbhit()) {
       sound(440);
       delay(1000);
       nosound();
       delay(1000);
    }
    fprintf(stderr,"\n");
}
